package de.uka.ilkd.key.rule.conditions;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.AnonymisingUpdateFactory;
import de.uka.ilkd.key.logic.LocationDescriptor;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.SVSubstitute;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.rule.IteratorOfObject;
import de.uka.ilkd.key.rule.ListOfObject;
import de.uka.ilkd.key.rule.MatchConditions;
import de.uka.ilkd.key.rule.VariableCondition;
import de.uka.ilkd.key.rule.inst.ListInstantiation;
import de.uka.ilkd.key.rule.inst.SVInstantiations;


public class NewDepOnAnonUpdates implements VariableCondition {
    
    private final SchemaVariable modifies, newFunctionListSV;

    
    public NewDepOnAnonUpdates(SchemaVariable modifies, SchemaVariable list){
        this.modifies= modifies;
        this.newFunctionListSV = list;
    }
    

    public boolean isSchemaVariableList(SchemaVariable sv){
        return newFunctionListSV.equals(sv);
    }
    
    
    public MatchConditions check(SchemaVariable var, SVSubstitute instCandidate,
				 MatchConditions matchCond, 
				 Services services) {

        MatchConditions result = matchCond;
        SVInstantiations inst = matchCond.getInstantiations();

        if (inst.getInstantiationEntry(modifies) == null || 
               (inst.getInstantiationEntry(newFunctionListSV) instanceof ListInstantiation)){
            return result;
        }
         
        ListOfObject locationList
            = (ListOfObject) inst.getInstantiation(modifies);
        LocationDescriptor[] locations 
                = new LocationDescriptor[locationList.size()];
        IteratorOfObject it = locationList.iterator();
        for(int i = 0; i < locations.length; i++) {
            locations[i] = (LocationDescriptor) it.next();
        }
        
        Function[] uninterpretedFunctions 
            = AnonymisingUpdateFactory.createUninterpretedFunctions(locations, 
                                                                    services);
        
        inst   = inst.addList(newFunctionListSV, uninterpretedFunctions);
        result = result.setInstantiations(inst);
      
        return result;
    }
    
    public String toString(){
        return "\\newDependingOnMod("+newFunctionListSV+","+modifies+")";
    }

    
    public boolean isModifiesSV(SchemaVariable var) {
        return modifies == var;
    }
}
